/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { products } from "../config/products.js";

@Component({
  imports:     [ CommonModule ],
  selector:    "app-products",
  styleUrl:    "./products.css",
  templateUrl: "./products.html",
})
export class Products {
  public view: (typeof products)[number]["category"] | "all"
    = "all";
  public products:         typeof products = [];
  public readonly filters: {
    wip:  boolean;
    prod: boolean;
    paid: boolean;
    free: boolean;
  } = {
      free: true,
      paid: true,
      prod: true,
      wip:  true,
    };

  public constructor() {
    this.selectCategory("all");
  }

  public selectCategory(
    category: (typeof products)[number]["category"] | "all",
  ): void {
    this.view = category;
    const sortedProducts = products.sort((a, b) => {
      return a.name.localeCompare(b.name);
    });
    if (this.view === "all") {
      this.products = sortedProducts;
      return;
    }
    this.products = sortedProducts.filter((product) => {
      return product.category === this.view;
    });
  }

  public toggleFilter(
    filter: "wip" | "prod" | "paid" | "free",
  ): void {
    this.filters[filter] = !this.filters[filter];
    this.applyFilters();
  }

  private applyFilters(): void {
    this.selectCategory(this.view);
    this.products = this.products.filter((product) => {
      if (!this.filters.wip && product.wip) {
        return false;
      }
      if (!this.filters.prod && !product.wip) {
        return false;
      }
      if (!this.filters.paid && product.premium) {
        return false;
      }
      if (!this.filters.free && !product.premium) {
        return false;
      }
      return true;
    });
  }
}
